// $Id$
/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.derivate;

import java.util.UUID;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * Editor input for the {@link DerivateView} which holds the currently selected derivate for which
 * the derivate hierarchy should be shown in the DerivateView.<br>
 * It also holds a {@link SpecimenOrObservationBase} which is the root of the hierarchy. (both may be the same object)
 * @author pplitzner
 * @date 25.11.2013
 *
 */
public class DerivateViewEditorInput implements IEditorInput{

    /**
     * The selected derivate {@link UUID}
     */
    private UUID derivateUUID;
    /**
     * The {@link UUID} of the root of the hierarchy (may be the same object as the derivate)
     */
    private UUID rootUUID;

    private final ConversationHolder conversationHolder;

    /**
     * Creates an editor input for the {@link DerivateView} with the currently selected derivate and the
     * corresponding {@link FieldUnit} (both may be the same object).
     * @param derivateUuid the {@link UUID} of the derivate for which the derivate hierarchy should be shown
     * @param rootUUID the root of the hierarchy
     */
    public DerivateViewEditorInput(UUID derivateUuid) {
        super();
        this.conversationHolder = CdmStore.createConversation();
        this.derivateUUID = derivateUuid;
        SpecimenOrObservationBase<?> derivate = CdmStore.getService(IOccurrenceService.class).load(derivateUuid);
        if(derivate instanceof FieldUnit){
            this.rootUUID = derivateUUID;
        }
        else if(derivate instanceof DerivedUnit){
            SpecimenOrObservationBase<?> topMostDerivate = EditorUtil.getTopMostDerivate(derivate);
            rootUUID = topMostDerivate==null?null:topMostDerivate.getUuid();
            if(rootUUID==null){
                rootUUID = derivateUUID;
            }
        }
        if(rootUUID==null){
            MessagingUtils.messageDialog("Failed initializing editor", DerivateViewEditorInput.class, "No root element found!");
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
     */
    @Override
    public Object getAdapter(Class adapter) {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#exists()
     */
    @Override
    public boolean exists() {
        return false;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getImageDescriptor()
     */
    @Override
    public ImageDescriptor getImageDescriptor() {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getName()
     */
    @Override
    public String getName() {
        return derivateUUID.toString();
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getPersistable()
     */
    @Override
    public IPersistableElement getPersistable() {
        return null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getToolTipText()
     */
    @Override
    public String getToolTipText() {
        return derivateUUID.toString();
    }

    public UUID getDerivateUUID() {
        return derivateUUID;
    }

    public UUID getRootElementUUID() {
        return rootUUID;
    }

    public void setRootAsDerivate(){
        this.derivateUUID = this.rootUUID;
    }

    public ConversationHolder getConversationHolder() {
        return conversationHolder;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((rootUUID == null) ? 0 : rootUUID.hashCode());
        return result;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        DerivateViewEditorInput other = (DerivateViewEditorInput) obj;
        if (rootUUID == null) {
            if (other.rootUUID != null) {
                return false;
            }
        } else if (!rootUUID.equals(other.rootUUID)) {
            return false;
        }
        return true;
    }

}
